<?php

namespace App\Repository;

use App\Entity\Snip;
use App\Entity\User;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;

/**
 * @extends ServiceEntityRepository<Snip>
 *
 * @method Snip|null find($id, $lockMode = null, $lockVersion = null)
 * @method Snip|null findOneBy(array $criteria, array $orderBy = null)
 * @method Snip[]    findAll()
 * @method Snip[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class SnipRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Snip::class);
    }

    public function save(Snip $entity, bool $flush = true): void
    {
        $this->getEntityManager()->persist($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function remove(Snip $entity, bool $flush = true): void
    {
        $this->getEntityManager()->remove($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function findByUser(User $user): array
    {
        $qb = $this->createQueryBuilder('s');
        $qb->where('s.createdBy = :user')
            ->setParameter('user', $user)
            ->orderBy('s.createdAt', 'DESC');

        return $qb->getQuery()->getResult();
    }

    public function findPublic(User $user): array
    {
        $qb = $this->createQueryBuilder('s');
        $qb->where('s.public = :public')
            ->andWhere('s.createdBy != :user')
            ->setParameter('public', true)
            ->setParameter('user', $user)
            ->orderBy('s.createdAt', 'DESC');

        return $qb->getQuery()->getResult();
    }
}
