<?php

namespace App\Service\SnipParser;

use App\Service\SnipParser\Stages\HtmlEscapeStage;
use App\Service\SnipParser\Stages\IncludeReferenceStage;
use App\Service\SnipParser\Stages\UrlReferenceStage;
use App\Service\SnipParser\Stages\ReplaceBlocksStage;
use App\Service\SnipParser\Stages\ReplaceStage;
use League\Pipeline\PipelineBuilder;

class Pipeline
{
    public function __construct(
        private readonly UrlReferenceStage     $referenceStage,
        private readonly IncludeReferenceStage $includeStage,
    ) {}

    public function parse(string $payload): string
    {
        $builder = new PipelineBuilder();
        $pipeline = $builder
            ->add(new HtmlEscapeStage())
            ->add(new ReplaceStage(PHP_EOL, '<br>'))
            ->add(new ReplaceBlocksStage('<pre><code class="hljs">', '</code></pre>', '```'))
            ->add(new ReplaceBlocksStage('<code class="hljs">', '</code>', '``'))
            ->add($this->referenceStage)
            ->add($this->includeStage)
            ->build()
        ;

        return $pipeline->process($payload);
    }

    public function clean(string $payload): string
    {
        return str_replace(
            ['```', '``'],
            '',
            $payload
        );
    }
}